///////////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 1999, Tres Seaver, Palladion Software. All rights reserved.
//  Orignal DUnit Copyright (C) 1999 Juancarlo Aez.
//  Original Java framework Copyright (C) 1999 by Kent Beck and Erich Gamma.
// 
//  LICENSE
//
//  Permission to reproduce and create derivative works from the
//  Software ("Software Derivative Works") is hereby granted to you
//  under the copyrights of Kent Beck, Erich Gamma, Juancarlo Aez,
//  and Tres Seaver (THE AUTHORS).  The authors also grant you the right
//  to distribute the Software and Software Derivative Works. 
//
//  The authors license the Software to you on an "AS IS" basis, without
//  warranty of any kind.  THE AUTHORS HEREBY EXPRESSLY DISCLAIMS ALL
//  WARRANTIES OR CONDITIONS, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT
//  NOT LIMITED TO, THE IMPLIED WARRANTIES OR CONDITIONS OF
//  MERCHANTABILITY, NON INFRINGEMENT AND FITNESS FOR A PARTICULAR
//  PURPOSE.  You are solely responsible for determining the
//  appropriateness of using the Software and assume all risks
//  associated with the use and distribution of this Software, including
//  but not limited to the risks of program errors, damage to or loss of
//  data, programs or equipment, and unavailability or interruption of
//  operations.  THE AUTHORS WILL NOT BE LIABLE FOR ANY DIRECT DAMAGES
//  OR FOR ANY SPECIAL, INCIDENTAL, OR INDIRECT DAMAGES OR FOR ANY
//  ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS),
//  EVEN IF THE AUTHORS HAD BEEN ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGE.  The authors will not be liable for the loss of, or damage
//  to, your records or data, or any damages claimed by you based on a
//  third party claim.  
//
//  You agree to distribute the Software and any Software Derivatives
//  under a license agreement that: 1) is sufficient to notify all
//  licensees of the Software and Software Derivatives that the authors
//  assume no liability for any claim that may arise regarding the
//  Software or Software Derivatives, and 2) that disclaims all
//  warranties, both express and implied, from the authors regarding the
//  Software and Software Derivatives.  (If you include this Agreement
//  with any distribution of the Software and Software Derivatives you
//  will have meet this requirement).  You agree that you will not
//  delete any copyright notices in the Software.
//
//  This Agreement is the exclusive statement of your rights in the
//  Software as provided by the authors.  Except for the licenses
//  granted to you in the second paragraph above, no other licenses are
//  granted hereunder, by estoppel, implication or otherwise.
//
///////////////////////////////////////////////////////////////////////////////
program ConsoleTest_DUnit_PEM;
//==============================================================================
//  Program:    ConsoleTest_DUnit_PEM
//
//  Purpose:
//
//  Demonstrate use of DUnit_PEM components: TTestResults/TTestCase/TTestSuite
//  in a console application.
//==============================================================================

uses  DUnit_PEM
    , Classes
    , SysUtils
    ;

type

  TTestCaseDriver = class

    procedure Log( fmt : String; args : array of const );     

    procedure DoTests( testSuite : TTestSuite );

    procedure TestResultTestStart(whichTest     : TTest
                                 ; res          : TTestResult);

    procedure TestResultTestEnd( whichTest      : TTest
                               ; res            : TTestResult);

    procedure TestResultTestError( whichTest    : TTest
                                 ; res          : TTestResult
                                 ; e            : Exception);

    procedure TestResultTestFailure( whichTest  : TTest
                                   ; res        : TTestResult
                                   ; e          : Exception);

    procedure TestSuiteBeginRun( testSuite: TTestSuite );

    procedure TestSuiteEndRun( testSuite: TTestSuite );

    procedure TestCaseSetup( testCase: TTestCase );

    procedure TestCaseTeardown( testCase: TTestCase );

    procedure TestCaseExecute( testCase: TTestCase );

    procedure AssertFalseTestCaseExecute( testCase: TTestCase );

    procedure LongValueTestCaseExecute( testCase: TTestCase );

    procedure DoubleValueTestCaseExecute( testCase: TTestCase );

    procedure AssignedTestCaseExecute( testCase: TTestCase );

    procedure UnassignedTestCaseExecute( testCase: TTestCase );

    procedure EqualityTestCaseExecute( testCase: TTestCase );

    procedure RandomExceptionTestCaseExecute( testCase: TTestCase );

    procedure DivByZeroTestCaseExecute( testCase: TTestCase );
  end;


procedure TTestCaseDriver.Log( fmt : String; args : array of const );
begin
    Writeln( Format( fmt, args ) );
end;

procedure TTestCaseDriver.DoTests( testSuite : TTestSuite );
var
    tr          : TTestResult;
    iFailure    : Integer;
    iError      : Integer;
begin
    Log( '----------------------------------------', [] );
    testSuite.Run;
    Log( '----------------------------------------', [] );

    tr := testSuite.TestResult;

    Log( 'Test cases completed: %d', [ tr.TestsRunCount ] );

    Log( 'Failures:        %d', [ tr.FailureCount ] );
    for iFailure := 0 to tr.FailureCount - 1 do
      with tr.Failures[ iFailure ] do
       Log( '* %s', [ Message ] );

    Log( 'Errors:          %d', [ tr.ErrorCount ] );
    for iError := 0 to tr.ErrorCount - 1 do
      with tr.Errors[ iError ] do
       Log( '* %s', [ Message ] );

    Log( '----------------------------------------', [] );

    Writeln( 'Hit <ENTER> to end the test.' );
    Readln;
end;

procedure TTestCaseDriver.TestResultTestStart(whichTest: TTest; res: TTestResult);
begin
  Log( '* Test case %s starting', [ whichTest.Name ] );
end;

procedure TTestCaseDriver.TestResultTestEnd(whichTest: TTest; res: TTestResult);
begin
  Log( '* Test case %s done', [ whichTest.Name ] );
end;

procedure TTestCaseDriver.TestResultTestError(whichTest: TTest; res: TTestResult;
  e: Exception);
begin
  Log( '* Test case %s error: %s', [ whichTest.Name, e.Message ] );
end;

procedure TTestCaseDriver.TestResultTestFailure(whichTest: TTest; res: TTestResult;
  e: Exception);
begin
  Log( '* Test case %s failure: %s', [ whichTest.Name, e.Message ] );
end;

procedure TTestCaseDriver.TestSuiteBeginRun(testSuite: TTestSuite);
begin
    Log( '*** Beginning suite: %s', [ testSuite.Name ] );
end;

procedure TTestCaseDriver.TestSuiteEndRun(testSuite: TTestSuite);
begin
    Log( '*** Ending suite: %s', [ testSuite.Name ] );
end;

procedure TTestCaseDriver.TestCaseSetup(testCase: TTestCase);
begin
    Log( '*** Setting up %s', [ testCase.Name ] );
end;

procedure TTestCaseDriver.TestCaseExecute(testCase: TTestCase);
begin
    Log( '*** Executing %s', [ testCase.Name ] );
end;

procedure TTestCaseDriver.TestCaseTeardown(testCase: TTestCase);
begin
    Log( '*** Tearing down %s', [ testCase.Name ] );
end;

procedure TTestCaseDriver.AssertFalseTestCaseExecute(testCase: TTestCase);
begin
    testCase.Assert( True, 'shouldnt happen' );
    testCase.Assert( False, 'deliberate' );
end;

procedure TTestCaseDriver.LongValueTestCaseExecute(testCase: TTestCase);
begin
    testCase.AssertLongValue( 42, 42, 'shouldnt happen' );
    testCase.AssertLongValue( 42, 0, 'deliberate' );
end;

procedure TTestCaseDriver.DoubleValueTestCaseExecute(testCase: TTestCase);
begin
    testCase.AssertDoubleValue( 42.00, 42.00, 0.001, 'shouldnt happen' );
    testCase.AssertDoubleValue( 42.00, 42.0005, 0.001, 'shouldnt happen' );
    testCase.AssertDoubleValue( 42.00, 42.02, 0.001, 'deliberate' );
end;

procedure TTestCaseDriver.AssignedTestCaseExecute(testCase: TTestCase);
var
    uninitialized : TObject;
begin
    uninitialized := Nil;
    testCase.AssertAssigned( testCase, 'shouldnt happen' );
    testCase.AssertAssigned( uninitialized, 'deliberate' );
end;

procedure TTestCaseDriver.UnassignedTestCaseExecute(testCase: TTestCase);
var
    uninitialized : TObject;
begin
    uninitialized := Nil;
    testCase.AssertUnassigned( uninitialized, 'shouldnt happen' );
    testCase.AssertUnassigned( testCase, 'deliberate' );
end;

procedure TTestCaseDriver.EqualityTestCaseExecute( testCase: TTestCase );
var
    clone   : TTestCase;
begin
    clone   := testCase;
    testCase.AssertIdentity( testCase, clone, 'shouldnt happen' );
    testCase.AssertIdentity( testCase, self, 'deliberate' );
end;

procedure TTestCaseDriver.RandomExceptionTestCaseExecute(testCase: TTestCase);
begin
    raise Exception.Create( 'deliberate' );
end;


procedure TTestCaseDriver.DivByZeroTestCaseExecute(testCase: TTestCase);
var
    y   : Integer;
begin
    y := 0;
    if ( 3 mod y ) > 0 then
        raise Exception.Create( 'shouldnt happen' );
end;

var
    driver                  : TTestCaseDriver;

    theTestResult           : TTestResult;
    mainTestSuite           : TTestSuite;
    basicTestCase           : TTestCase;

    subTestSuite            : TTestSuite;
    subTestCase1            : TTestCase;
    subTestCase2            : TTestCase;

    AssertFalseTestCase     : TTestCase;
    LongValueTestCase       : TTestCase;
    DoubleValueTestCase     : TTestCase;
    AssignedTestCase        : TTestCase;
    UnassignedTestCase      : TTestCase;
    EqualityTestCase        : TTestCase;
    RandomExceptionTestCase : TTestCase;
    DivByZeroTestCase       : TTestCase;

begin
    driver                  := TTestCaseDriver.Create;

    theTestResult           := TTestResult.Create( nil );
    with theTestResult do
    begin
        Name                := 'theTestResult';
        OnTestStart         := driver.TestResultTestStart;
        OnTestError         := driver.TestResultTestError;
        OnTestFailure       := driver.TestResultTestFailure;
        OnTestEnd           := driver.TestResultTestEnd;
    end;

    mainTestSuite           := TTestSuite.Create( nil );
    with mainTestSuite do
    begin
        Name                := 'mainTestSuite';
        OnBeginRun          := driver.TestSuiteBeginRun;
        OnEndRun            := driver.TestSuiteEndRun;
        TestResult          := theTestResult;
    end;

    basicTestCase           := TTestCase.Create( nil );
    with basicTestCase do
    begin
        Name                := 'basicTestCase';
        OnSetup             := driver.TestCaseSetup;
        OnExecute           := driver.TestCaseExecute;
        OnTeardown          := driver.TestCaseTeardown;
        Parent              := mainTestSuite;
    end;

    subTestSuite            := TTestSuite.Create( nil );
    with subTestSuite do
    begin
        Name                := 'subTestSuite';
        OnBeginRun          := driver.TestSuiteBeginRun;
        OnEndRun            := driver.TestSuiteEndRun;
        TestResult          := theTestResult;
        Parent              := mainTestSuite;
    end;

    subTestCase1            := TTestCase.Create( nil );
    with subTestCase1 do
    begin
        Name                := 'subTestCase1';
        OnSetup             := driver.TestCaseSetup;
        OnExecute           := driver.TestCaseExecute;
        OnTeardown          := driver.TestCaseTeardown;
        Parent              := subTestSuite;
    end;

    subTestCase2            := TTestCase.Create( nil );
    with subTestCase2 do
    begin
        Name                := 'subTestCase2';
        OnSetup             := driver.TestCaseSetup;
        OnExecute           := driver.TestCaseExecute;
        OnTeardown          := driver.TestCaseTeardown;
        Parent              := subTestSuite;
    end;

    AssertFalseTestCase     := TTestCase.Create( nil );
    with AssertFalseTestCase do
    begin
        Name                := 'AssertFalseTestCase';
        OnExecute           := driver.AssertFalseTestCaseExecute;
        Parent              := mainTestSuite;
    end;

    LongValueTestCase       := TTestCase.Create( nil );
    with LongValueTestCase do
    begin
        Name                := 'LongValueTestCase';
        OnExecute           := driver.LongValueTestCaseExecute;
        Parent              := mainTestSuite;
    end;

    DoubleValueTestCase     := TTestCase.Create( nil );
    with DoubleValueTestCase do
    begin
        Name                := 'DoubleValueTestCase';
        OnExecute           := driver.DoubleValueTestCaseExecute;
        Parent              := mainTestSuite;
    end;

    AssignedTestCase        := TTestCase.Create( nil );
    with AssignedTestCase do
    begin
        Name                := 'AssignedTestCase';
        OnExecute           := driver.AssignedTestCaseExecute;
        Parent              := mainTestSuite;
    end;

    UnassignedTestCase      := TTestCase.Create( nil );
    with UnassignedTestCase do
    begin
        Name                := 'UnassignedTestCase';
        OnExecute           := driver.UnassignedTestCaseExecute;
        Parent              := mainTestSuite;
    end;

    EqualityTestCase        := TTestCase.Create( nil );
    with EqualityTestCase do
    begin
        Name                := 'EqualityTestCase';
        OnExecute           := driver.EqualityTestCaseExecute;
        Parent              := mainTestSuite;
    end;

    RandomExceptionTestCase := TTestCase.Create( nil );
    with RandomExceptionTestCase do
    begin
        Name                := 'RandomExceptionTestCase';
        OnExecute           := driver.RandomExceptionTestCaseExecute;
        Parent              := mainTestSuite;
    end;

    DivByZeroTestCase       := TTestCase.Create( nil );
    with DivByZeroTestCase do
    begin
        Name                := 'DivByZeroTestCase';
        OnExecute           := driver.DivByZeroTestCaseExecute;
        Parent              := mainTestSuite;
    end;

    driver.doTests( mainTestSuite );
end.
