///////////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 1999, Tres Seaver, Palladion Software. All rights reserved.
//  Orignal DUnit Copyright (C) 1999 Juancarlo Aez.
//  Original Java framework Copyright (C) 1999 by Kent Beck and Erich Gamma.
// 
//  LICENSE
//
//  Permission to reproduce and create derivative works from the
//  Software ("Software Derivative Works") is hereby granted to you
//  under the copyrights of Kent Beck, Erich Gamma, Juancarlo Aez,
//  and Tres Seaver (THE AUTHORS).  The authors also grant you the right
//  to distribute the Software and Software Derivative Works. 
//
//  The authors license the Software to you on an "AS IS" basis, without
//  warranty of any kind.  THE AUTHORS HEREBY EXPRESSLY DISCLAIMS ALL
//  WARRANTIES OR CONDITIONS, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT
//  NOT LIMITED TO, THE IMPLIED WARRANTIES OR CONDITIONS OF
//  MERCHANTABILITY, NON INFRINGEMENT AND FITNESS FOR A PARTICULAR
//  PURPOSE.  You are solely responsible for determining the
//  appropriateness of using the Software and assume all risks
//  associated with the use and distribution of this Software, including
//  but not limited to the risks of program errors, damage to or loss of
//  data, programs or equipment, and unavailability or interruption of
//  operations.  THE AUTHORS WILL NOT BE LIABLE FOR ANY DIRECT DAMAGES
//  OR FOR ANY SPECIAL, INCIDENTAL, OR INDIRECT DAMAGES OR FOR ANY
//  ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS),
//  EVEN IF THE AUTHORS HAD BEEN ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGE.  The authors will not be liable for the loss of, or damage
//  to, your records or data, or any damages claimed by you based on a
//  third party claim.
//
//  You agree to distribute the Software and any Software Derivatives
//  under a license agreement that: 1) is sufficient to notify all
//  licensees of the Software and Software Derivatives that the authors
//  assume no liability for any claim that may arise regarding the
//  Software or Software Derivatives, and 2) that disclaims all
//  warranties, both express and implied, from the authors regarding the
//  Software and Software Derivatives.  (If you include this Agreement
//  with any distribution of the Software and Software Derivatives you
//  will have meet this requirement).  You agree that you will not
//  delete any copyright notices in the Software.
//
//  This Agreement is the exclusive statement of your rights in the
//  Software as provided by the authors.  Except for the licenses
//  granted to you in the second paragraph above, no other licenses are
//  granted hereunder, by estoppel, implication or otherwise.
//
///////////////////////////////////////////////////////////////////////////////
unit fmMain;
//==============================================================================
//  Unit:       fmMain
//
//  Purpose:
//
//  Demonstrate use of DUnit_PEM components: TTestResults/TTestCase/TTestSuite.
//==============================================================================

interface

uses  Windows
    , Messages
    , SysUtils
    , Classes
    , Graphics
    , Controls
    , Forms
    , Dialogs
    , StdCtrls
    , ExtCtrls
    , DUnit_PEM
    ;

type

  TfrmMain = class(TForm)

    Panel1: TPanel;
    Memo1: TMemo;
    TestSuite1: TTestSuite;
    TestCase1: TTestCase;
    TestResult1: TTestResult;
    TestSuite2: TTestSuite;
    TestCase2: TTestCase;
    TestCase3: TTestCase;
    AssertFalseTestCase: TTestCase;
    LongValueTestCase: TTestCase;
    DoubleValueTestCase: TTestCase;
    AssignedTestCase: TTestCase;
    UnassignedTestCase: TTestCase;
    EqualityTestCase: TTestCase;
    RandomExceptionTestCase: TTestCase;
    DivByZeroTestCase: TTestCase;
    Button1: TButton;

    procedure RunTests(Sender: TObject);
    procedure TestResult1TestStart(whichTest: TTest; res: TTestResult);
    procedure TestResult1TestEnd(whichTest: TTest; res: TTestResult);
    procedure TestResult1TestError(whichTest: TTest; res: TTestResult;
      e: Exception);
    procedure TestResult1TestFailure(whichTest: TTest; res: TTestResult;
      e: Exception);
    procedure TestCase1Setup(testCase: TTestCase);
    procedure TestCase1Teardown(testCase: TTestCase);
    procedure TestCase1Execute(testCase: TTestCase);
    procedure TestSuite1BeginRun(testSuite: TTestSuite);
    procedure TestSuite1EndRun(testSuite: TTestSuite);
    procedure AssertFalseTestCaseExecute(testCase: TTestCase);
    procedure LongValueTestCaseExecute(testCase: TTestCase);
    procedure DoubleValueTestCaseExecute(testCase: TTestCase);
    procedure AssignedTestCaseExecute(testCase: TTestCase);
    procedure UnassignedTestCaseExecute(testCase: TTestCase);
    procedure EqualityTestCaseExecute(testCase: TTestCase);
    procedure RandomExceptionTestCaseExecute(testCase: TTestCase);
    procedure DivByZeroTestCaseExecute(testCase: TTestCase);
  private
    procedure Log( fmt : String; args : array of const );
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

procedure TfrmMain.Log( fmt : String; args : array of const );
begin
    Memo1.Lines.Add( Format( fmt, args ) );
end;

procedure TfrmMain.RunTests(Sender: TObject);
var
    iFailure    : Integer;
    iError      : Integer;
begin
    Log( '----------------------------------------', [] );
    TestSuite1.Run;
    Log( '----------------------------------------', [] );
    Log( 'Test cases completed: %d', [ TestResult1.TestsRunCount ] );
    Log( 'Failures:        %d', [ TestResult1.FailureCount ] );
    for iFailure := 0 to TestResult1.FailureCount - 1 do
      with TestResult1.Failures[ iFailure ] do
       Log( '* %s', [ Message ] );
    Log( 'Errors:          %d', [ TestResult1.ErrorCount ] );
    for iError := 0 to TestResult1.ErrorCount - 1 do
      with TestResult1.Errors[ iError ] do
       Log( '* %s', [ Message ] );
    Log( '----------------------------------------', [] );
end;

procedure TfrmMain.TestResult1TestStart(whichTest: TTest; res: TTestResult);
begin
  Log( '* Test case %s starting', [ whichTest.Name ] );
end;

procedure TfrmMain.TestResult1TestEnd(whichTest: TTest; res: TTestResult);
begin
  Log( '* Test case %s done', [ whichTest.Name ] );
end;

procedure TfrmMain.TestResult1TestError(whichTest: TTest; res: TTestResult;
  e: Exception);
begin
  Log( '* Test case %s error: %s', [ whichTest.Name, e.Message ] );
end;

procedure TfrmMain.TestResult1TestFailure(whichTest: TTest; res: TTestResult;
  e: Exception);
begin
  Log( '* Test case %s failure: %s', [ whichTest.Name, e.Message ] );
end;

procedure TfrmMain.TestCase1Setup(testCase: TTestCase);
begin
    Log( '*** Setting up %s', [ testCase.Name ] );
end;

procedure TfrmMain.TestCase1Teardown(testCase: TTestCase);
begin
    Log( '*** Tearing down %s', [ testCase.Name ] );
end;

procedure TfrmMain.TestCase1Execute(testCase: TTestCase);
begin
    Log( '*** Executing %s', [ testCase.Name ] );
end;

procedure TfrmMain.TestSuite1BeginRun(testSuite: TTestSuite);
begin
    Log( '*** Beginning suite: %s', [ testSuite.Name ] );
end;

procedure TfrmMain.TestSuite1EndRun(testSuite: TTestSuite);
begin
    Log( '*** Ending suite: %s', [ testSuite.Name ] );
end;

procedure TfrmMain.AssertFalseTestCaseExecute(testCase: TTestCase);
begin
    testCase.Assert( True, 'shouldnt happen' );
    testCase.Assert( False, 'deliberate' );
end;

procedure TfrmMain.LongValueTestCaseExecute(testCase: TTestCase);
begin
    testCase.AssertLongValue( 42, 42, 'shouldnt happen' );
    testCase.AssertLongValue( 42, 0, 'deliberate' );
end;

procedure TfrmMain.DoubleValueTestCaseExecute(testCase: TTestCase);
begin
    testCase.AssertDoubleValue( 42.00, 42.00, 0.001, 'shouldnt happen' );
    testCase.AssertDoubleValue( 42.00, 42.0005, 0.001, 'shouldnt happen' );
    testCase.AssertDoubleValue( 42.00, 42.02, 0.001, 'deliberate' );
end;

procedure TfrmMain.AssignedTestCaseExecute(testCase: TTestCase);
var
    uninitialized : TObject;
begin
    uninitialized := Nil;
    testCase.AssertAssigned( testCase, 'shouldnt happen' );
    testCase.AssertAssigned( uninitialized, 'deliberate' );
end;

procedure TfrmMain.UnassignedTestCaseExecute(testCase: TTestCase);
var
    uninitialized : TObject;
begin
    uninitialized := Nil;
    testCase.AssertUnassigned( uninitialized, 'shouldnt happen' );
    testCase.AssertUnassigned( testCase, 'deliberate' );
end;

procedure TfrmMain.EqualityTestCaseExecute(testCase: TTestCase);
begin
    testCase.AssertIdentity( testCase.Parent, TestSuite1, 'shouldnt happen' );
    testCase.AssertIdentity( testCase.Parent, TestSuite2, 'deliberate' );
end;

procedure TfrmMain.RandomExceptionTestCaseExecute(testCase: TTestCase);
begin
    raise Exception.Create( 'deliberate' );
end;


procedure TfrmMain.DivByZeroTestCaseExecute(testCase: TTestCase);
var
    x   : Integer;
    y   : Integer;
begin
    y := 0;
    if ( 3 mod y ) > 0 then
        raise Exception.Create( 'shouldnt happen' );
end;

end.
